/*
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 * bdc_time.h
 *
 * Copyright (C) 2011-2022, University of Vienna and Vienna Institute for Nature Conservation & Analyses, Andreas Gattringer.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#ifndef BDC_TIME_H
#define BDC_TIME_H


#include <stdint.h>

#ifdef BDC_ON_WINDOWS
#include <winsock.h>
#else
#include <sys/time.h>
#endif
#include <time.h>
struct bdc_time {

// monotonic time stamps, used for total elapsed time
#ifdef BDC_ON_WINDOWS
        LARGE_INTEGER clock_monotonic;
#else
        struct timespec clock_monotonic;
#endif

        struct timeval current_time;
};

double seconds_monotonic_since(struct bdc_time *start_time);

void time_now(struct bdc_time *time_info);

double get_elapsed_seconds_monotonic(struct bdc_time *start_time, struct bdc_time *end_time);

void get_time(struct timeval *tv);

struct bdc_timer {
        clock_t start_clock;
        struct timeval start_timeval;
        double time_normal;
        double time_real;
};



struct bdc_timer start_new_timer(void);


void start_timer(struct bdc_timer *t);

void stop_timer(struct bdc_timer *t);
#endif //BDC_TIME_H
